@@ -4,9 +4,11 @@
 #include <stdlib.h>
 #include <unistd.h>
 #include <string.h>
-#include <getopt.h>
+#ifndef _AIX
+# include <getopt.h>
+#endif
 #include <ctype.h>
-#ifndef WIN32
+#if !defined(WIN32) && !defined(_AIX)
 # include <err.h>
 #endif
 #include "json.h"
@@ -44,9 +46,36 @@
 
 static JsonNode *pile;		/* pile of nested objects/arrays */
 
-#ifdef _WIN32
-# define err(n, s)	{ fprintf(stderr, s); exit(n); }
-# define errx(n, f, a)	{ fprintf(stderr, f, a); exit(n); }
+#if defined(_WIN32) || defined(_AIX)
+#include <errno.h>
+#include <stdarg.h>
+static inline void err(int eval, const char *fmt, ...) {
+	int errnum = errno;
+
+	va_list ap;
+	va_start(ap, fmt);
+
+	fprintf(stderr, "jo: ");
+	vfprintf(stderr, fmt, ap);
+	fprintf(stderr, ": %s\n", strerror(errnum));
+
+	va_end(ap);
+	exit(eval);
+}
+static inline void errx(int eval, const char *fmt, ...) {
+	va_list ap;
+	va_start(ap, fmt);
+
+	fprintf(stderr, "jo: ");
+	vfprintf(stderr, fmt, ap);
+	fprintf(stderr, "\n");
+
+	va_end(ap);
+	exit(eval);
+}
+#endif
+
+#if defined(_WIN32) && !defined(fseeko)
 # define fseeko	fseek
 # define ftello	ftell
 #endif
@@ -57,6 +86,8 @@ static JsonNode *pile;		/* pile of nested objects/arrays */
 #define TAG_FLAG_NUMBER   (TAG_TO_FLAGS(JSON_NUMBER))
 #define COERCE_MASK       (TAG_FLAG_BOOL | TAG_FLAG_STRING | TAG_FLAG_NUMBER)
 
+#define debug(format, args...) fprintf (stderr, format, args)
+
 JsonTag flags_to_tag(int flags) {
 	return flags / (FLAG_MASK + 1);
 }
@@ -101,11 +132,45 @@ void json_copy_to_object(JsonNode * obj, JsonNode * object_or_array, int clobber
 	}
 }
 
-char *slurp_file(const char* filename, size_t *out_len, bool fold_newlines)
+char *slurp(FILE *fp, off_t bufblk_sz, int eos_char, size_t *out_len, bool fold_newlines)
 {
 	char *buf;
 	int i = 0;
-	int ch;
+	int ch = EOF;
+	off_t buffer_len = bufblk_sz;
+
+	if ((buf = malloc(buffer_len)) == NULL) {
+		i = -1;
+	} else {
+		while ((ch = fgetc(fp)) != eos_char && ch != EOF) {
+			if (i == (buffer_len - 1)) {
+				buffer_len += bufblk_sz;
+				if ((buf = realloc(buf, buffer_len)) == NULL) {
+					i = -1;
+					break;
+				}
+			}
+			if (ch != '\n' || !fold_newlines) {
+				buf[i++] = ch;
+			}
+		}
+	}
+	if (ch == EOF && i <= 0) {
+		/* EOF at first read */
+		if (buf) {
+			free(buf);
+			buf = NULL;
+		}
+	} else if (buf) {
+		buf[i] = 0;
+	}
+	*out_len = i;
+	return buf;
+}
+
+char *slurp_file(const char* filename, size_t *out_len, bool fold_newlines)
+{
+	char *buf;
 	off_t buffer_len;
 	FILE *fp;
 	bool use_stdin = strcmp(filename, "-") == 0;
@@ -125,24 +190,23 @@ char *slurp_file(const char* filename, size_t *out_len, bool fold_newlines)
 		fseeko(fp, 0, SEEK_SET);
 	}
 
-	if ((buf = malloc(buffer_len)) == NULL) {
+	buf = slurp(fp, buffer_len, EOF, out_len, fold_newlines);
+	if (*out_len == (size_t)-1) {
 		errx(1, "File %s is too large to be read into memory", filename);
 	}
-	while ((ch = fgetc(fp)) != EOF) {
-		if (i == (buffer_len - 1)) {
-			buffer_len += SLURP_BLOCK_SIZE;
-			if ((buf = realloc(buf, buffer_len)) == NULL) {
-				errx(1, "File %s is too large to be read into memory", filename);
-			}
-		}
-		if (ch != '\n' || !fold_newlines) {
-			buf[i++] = ch;
-		}
-	}
 	if (!use_stdin) fclose(fp);
-	buf[i] = 0;
-	*out_len = i;
-	return (buf);
+	return buf;
+}
+
+char *slurp_line(FILE *fp, size_t *out_len)
+{
+	char *buf;
+
+	buf = slurp(fp, SLURP_BLOCK_SIZE, '\n', out_len, false);
+	if (*out_len == (size_t)-1) {
+		errx(1, "Line too large to be read into memory");
+	}
+	return buf;
 }
 
 JsonNode *jo_mknull(JsonTag type) {
@@ -336,7 +400,7 @@ JsonNode *boolnode(char *str)
 
 int usage(char *prog)
 {
-	fprintf(stderr, "Usage: %s [-a] [-B] [-D] [-d keydelim] [-p] [-e] [-n] [-v] [-V] [-f file] [--] [-s|-n|-b] [word...]\n", prog);
+	fprintf(stderr, "Usage: %s [-a] [-B] [-D] [-d keydelim] [-p] [-e] [-n] [-o outfile] [-v] [-V] [-f file] [--] [-s|-n|-b] [word...]\n", prog);
 	fprintf(stderr, "\tword is key=value or key@value\n");
 	fprintf(stderr, "\t-a creates an array of words\n");
 	fprintf(stderr, "\t-B disable boolean true/false/null detection\n");
@@ -348,6 +412,7 @@ int usage(char *prog)
 	fprintf(stderr, "\t-s coerce type guessing to string\n");
 	fprintf(stderr, "\t-b coerce type guessing to bool\n");
 	fprintf(stderr, "\t-n coerce type guessing to number\n");
+	fprintf(stderr, "\t-o output to the given file\n");
 	fprintf(stderr, "\t-v show version\n");
 	fprintf(stderr, "\t-V show version in JSON\n");
 
@@ -504,7 +569,7 @@ char* utf8_from_locale(const char *str, size_t len)
 	if (len == 0) {
 		return strdup("");
 	}
-	if (len == -1) {
+	if (len == (size_t)-1) {
 		len = strlen(str);
 	}
 	wcssize = MultiByteToWideChar(GetACP(), 0, str, len,  NULL, 0);
@@ -537,7 +602,7 @@ char* locale_from_utf8(const char *utf8, size_t len)
 	if (len == 0) {
 		return strdup("");
 	}
-	if (len == -1) {
+	if (len == (size_t)-1) {
 		len = strlen(utf8);
 	}
 	wcssize = MultiByteToWideChar(CP_UTF8, 0, utf8, len,  NULL, 0);
@@ -595,10 +660,13 @@ int main(int argc, char **argv)
 {
 	int c, key_delim = 0;
 	bool showversion = false;
-	char *kv, *js_string, *progname, buf[BUFSIZ], *p;
+	char *kv, *js_string, *progname, *buf, *p;
 	char *in_file = NULL, *in_str;
+	char *out_file = NULL;
+	FILE *out = stdout;
 	size_t in_len = 0;
-	int ttyin = isatty(fileno(stdin)), ttyout = isatty(fileno(stdout));
+	int ttyin = isatty(fileno(stdin));
+	int ttyout = isatty(fileno(stdout));
 	int flags = 0;
 	JsonNode *json, *op;
 
@@ -610,7 +678,7 @@ int main(int argc, char **argv)
 
 	progname = (progname = strrchr(*argv, '/')) ? progname + 1 : *argv;
 
-	while ((c = getopt(argc, argv, "aBDd:f:hpenvV")) != EOF) {
+	while ((c = getopt(argc, argv, "aBDd:f:hpeno:vV")) != EOF) {
 		switch (c) {
 			case 'a':
 				flags |= FLAG_ARRAY;
@@ -639,6 +707,9 @@ int main(int argc, char **argv)
 			case 'n':
 				flags |= FLAG_SKIPNULLS;
 				break;
+			case 'o':
+				out_file = optarg;
+				break;
 			case 'v':
 				printf("jo %s\n", PACKAGE_VERSION);
 				exit(0);
@@ -683,12 +754,11 @@ int main(int argc, char **argv)
 		if (flags & FLAG_NOSTDIN) {
 			return(0);
 		}
-		while (fgets(buf, sizeof(buf), stdin) != NULL) {
-			if (buf[strlen(buf) - 1] == '\n')
-				buf[strlen(buf) - 1] = 0;
+		while ((buf = slurp_line(stdin, &in_len)) != NULL && in_len > 0) {
 			p = ttyin ? utf8_from_locale(buf, -1) : buf;
 			append_kv(json, flags, key_delim, p);
 			if (ttyin) utf8_free(p);
+			if (buf) free(buf);
 		}
 	} else {
 		while ((kv = *argv++)) {
@@ -747,8 +817,16 @@ int main(int argc, char **argv)
 		exit(2);
 	}
 
+	if (out_file != NULL) {
+		out = fopen(out_file, "w");
+		if (out == NULL) {
+			perror(out_file);
+			errx(1, "Cannot open %s for writing", out_file);
+		}
+		ttyout = isatty(fileno(out));
+	}
 	p = ttyout ? locale_from_utf8(js_string, -1) : js_string;
-	printf("%s\n", p);
+	fprintf(out, "%s\n", p);
 	if (ttyout) locale_free(p);
 	free(js_string);
 	json_delete(json);
